#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "funcions_eclipses.c"

const double PI=3.14159265;
const double MT=5.9742e24;
const double ML=7.3483e22;
const double MS=1.9891e30;

const double RS=696000.0;
const double RT=6378.140;
const double RTp=6356.755;
const double RL=1738.0;

int N=0;
int B=1;

int main(void) {

	int i, j, t, any_ini=2002, any_fin=2018, tipus=0;
	double Time, Tfin, inc, ini_ecl, fin_ecl;
	char nom[30], ephem[30];
	strcpy(nom,"ecl_sol_2002_2018.txt");
	strcpy(ephem,"ephem_1600_2200.405");
	FILE *fit;
	data dat;
	eclipse *list;
	
	if(B) {
		
		printf("Escriu l'any inicial:\n");
		scanf("%d",&any_ini);
		
		printf("Escriu l'any final:\n");
		scanf("%d",&any_fin);
		if(any_fin<any_ini) {
			printf("Error a l'escriure l'any final\n");
			return -1;
		}
		
		printf("Eclipse solar (0) o lunar (1):\n");
		scanf("%d",&tipus);
		if(tipus!=0&&tipus!=1) {
			printf("Error\n");
			return -1;
		}
		
		printf("Escriu el fitxer en binari utilitzat per obtenir les posicions:\n");
		scanf("%s",&ephem);

		printf("Escriu el fitxer de sortida:\n");
		scanf("%s",&nom);
	}
	
	Initialize_Ephemeris(ephem);
	fit=fopen(nom,"w");
	
	Time=Gregorian_to_Julian( any_ini, 1, 1, 0, 0, 0.0);
	Tfin=Gregorian_to_Julian( any_fin+1, 1, 1, 0, 0, 0.0);
		
	list=malloc(6*(any_fin-any_ini)*sizeof(eclipse));

	inc=(double)1/(24*4);
	t=0;
	if(tipus==0) {
		fprintf(fit,"Tipus Ecl. #    Temps inicial     #     Temps final     #      Lloc inicial     #     Lloc final    \n");
		fprintf(fit,"#####################################################################################################");
	}
	else {
		fprintf(fit,"Tipus Ecl. #    Temps inicial     #     Temps final     \n");
		fprintf(fit,"#######################################################");
	}
	for(Time+inc; Time<Tfin; Time=Time+inc) {
		if(tipus==0) {
			t=eclipse_solar(Time);
		}
		else {
			t=eclipse_lunar(Time);
		}
		if(t>0) {
			gdcon(Time, &dat);
			if(tipus==0) {
				list[N]=dades_eclipse_solar(dat.any, dat.mes, dat.dia);
			}
			else {
				list[N]=dades_eclipse_lunar(dat.any, dat.mes, dat.dia);
			}
			Time=Time+2;
			N++;
		}
	}

	if(tipus==0) {
		for(i=0;i<N;i++) {
			fprintf(fit,"\n\n  Parcial  ");
			ini_ecl=list[i].temps[0];
			gdcon(ini_ecl,&dat);
			fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
			fin_ecl=list[i].temps[list[i].n];
			gdcon(fin_ecl,&dat);
			fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
			fprintf(fit, "# %10.5f %9.5f ", list[i].punt_contacte[0][1]*180/PI, list[i].punt_contacte[0][2]*180/PI);
			fprintf(fit, "# %10.5f %9.5f ", list[i].punt_contacte[list[i].n][1]*180/PI, list[i].punt_contacte[list[i].n][2]*180/PI);
			for(j=1;j<(list[i].n-1);j++) {
				if(list[i].tipus[j]==2) {
					fprintf(fit,"\n    Total  ");
				}
				else {
					fprintf(fit,"\n   Anular  ");
				}
				gdcon(list[i].temps[j],&dat);
				fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
				gdcon(list[i].temps[j+1],&dat);
				fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
				fprintf(fit, "# %10.5f %9.5f ", list[i].punt_contacte[j][1]*180/PI, list[i].punt_contacte[j][2]*180/PI);
				fprintf(fit, "# %10.5f %9.5f ", list[i].punt_contacte[j+1][1]*180/PI, list[i].punt_contacte[j+1][2]*180/PI);
			}
		}
	}
	else {
		for(i=0;i<N;i++) {
			fprintf(fit,"\n\n Penombra  ");
			ini_ecl=list[i].temps[0];
			gdcon(ini_ecl,&dat);
			fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
			fin_ecl=list[i].temps[list[i].n];
			gdcon(fin_ecl,&dat);
			fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
			
			if(list[i].n>2) {
				fprintf(fit,"\n  Parcial  ");
				ini_ecl=list[i].temps[1];
				gdcon(ini_ecl,&dat);
				fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
				fin_ecl=list[i].temps[list[i].n-1];
				gdcon(fin_ecl,&dat);
				fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
				
				if(list[i].n>4) {
					fprintf(fit,"\n   Total   ");
					ini_ecl=list[i].temps[2];
					gdcon(ini_ecl,&dat);
					fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
					fin_ecl=list[i].temps[list[i].n-2];
					gdcon(fin_ecl,&dat);
					fprintf(fit,"# %2d/%2d/%4d %2d:%5.2f ",dat.dia,dat.mes,dat.any,dat.hora,(double)dat.min+dat.seg/60.0);
					
				}
			}
		}
	}
		
	free(list);
	fclose(fit);
	return 0;
}
