#include <math.h>
#include "funcions_posicio.c"
#include "eclipse.h"
#include "temps.c"

extern const double RS, RT, RTp, RL;

double max_eclip;
double t_max_eclip;

int eclipse_solar(double Time) {
	double S[3], T[3], L[3], V[3]; //punts
	double ST[3], SL[3], TL[3]; //vectors unitaris
	double dSL, dST, dLT, dSV, dVT; //distancies
	double TSL, STL, aT, bT, aL, bL; //angles (a angle recta tangent ext. i r, i b angle recta tang. int. i r. On r s la recta entre els centres)
	int i;
	
	pos_STL(Time,S,T,L);
	
	dST=sqrt(pow(T[0]-S[0],2)+pow(T[1]-S[1],2)+pow(T[2]-S[2],2));
	dSL=sqrt(pow(L[0]-S[0],2)+pow(L[1]-S[1],2)+pow(L[2]-S[2],2));
	dLT=sqrt(pow(T[0]-L[0],2)+pow(T[1]-L[1],2)+pow(T[2]-L[2],2));
	
	for(i=0;i<3;i++) {
		ST[i]=(T[i]-S[i])/dST;
		SL[i]=(L[i]-S[i])/dSL;
		TL[i]=(L[i]-T[i])/dLT;
	}
	
	STL=acos(-(ST[0]*TL[0]+ST[1]*TL[1]+ST[2]*TL[2]));
	if(STL>(PI/6.0)) {
		return 0;
	}
	
	TSL=acos(ST[0]*SL[0]+ST[1]*SL[1]+ST[2]*SL[2]);
	aT=acos((RS-RT)/dST);
	bT=acos((RS+RT)/dST);
	aL=acos((RS-RL)/dSL);
	bL=acos((RS+RL)/dSL);
	
	if((TSL+bL)>aT) {
		return 0;
	}
	if(((TSL+aL)>aT)&&((TSL+bT)>aL)) {
		return 1;
	}
	
	dSV=dSL*RS/(RS-RL);
	for(i=0;i<3;i++) {
		V[i]=S[i]+dSV*SL[i];
	}
	dVT=sqrt(pow(T[0]-V[0],2)+pow(T[1]-V[1],2)+pow(T[2]-V[2],2));
	if((dVT<=RT)||(dSV>=(sqrt(pow(dST,2)+pow(RT,2))))) {
		return 2;
	}
	return 3;
}

int eclipse_lunar(double Time) {
	double S[3], T[3], L[3]; //punts
	double ST[3], SL[3], TL[3]; //vectors unitaris
	double dSL, dST, dLT, dSV, dVL; //distancies
	double TSL, SLT, aT, bT, aL, bL; //angles (a angle recta tangent ext. i r, i b angle recta tang. int. i r. On r s la recta entre els centres)
	int i;
	
	pos_STL(Time,S,T,L);
	
	dST=sqrt(pow(T[0]-S[0],2)+pow(T[1]-S[1],2)+pow(T[2]-S[2],2));
	dSL=sqrt(pow(L[0]-S[0],2)+pow(L[1]-S[1],2)+pow(L[2]-S[2],2));
	dLT=sqrt(pow(T[0]-L[0],2)+pow(T[1]-L[1],2)+pow(T[2]-L[2],2));
	
	for(i=0;i<3;i++) {
		ST[i]=(T[i]-S[i])/dST;
		SL[i]=(L[i]-S[i])/dSL;
		TL[i]=(L[i]-T[i])/dLT;
	}
	
	SLT=acos(SL[0]*TL[0]+SL[1]*TL[1]+SL[2]*TL[2]);
	if(SLT>(PI/6.0)) {
		return 0;
	}
	
	TSL=acos(ST[0]*SL[0]+ST[1]*SL[1]+ST[2]*SL[2]);
	aT=acos((RS-RT)/dST);
	bT=acos((RS+RT)/dST);
	aL=acos((RS-RL)/dSL);
	bL=acos((RS+RL)/dSL);
	
	if((TSL+bT)>aL) {
		return 0;
	}
	if((TSL+bL)>aT) {
		return 1;
	}
	if((TSL+aL)>aT) {
		return 2;
	}
	return 3;
}

//t=0: inici o final eclipse parcial
//t=1: inici o final eclipse total o anular
//t=2: pas entre eclipse total i anular (punt V)

void contacte_eclip(double Time, double P[3], int t) {
	int i;
	double S[3], T[3], L[3];
	double ST[3], SL[3], Q[3];
	double dST, dSL, cos_a, sin_a, ang, aux;
	
	pos_STL(Time,S,T,L);
	
	vect_unit(S,L,SL);
	if(t!=2) {
		vect_unit(S,T,ST);
		prod_vect(ST,SL,Q);
		prod_vect(Q,ST,P);
		norm(P);
		dST=dist(S,T);
		if(t==0) {
			cos_a=(RS-RT)/dST;
		}
		else if(t==1) {
			cos_a=-(RS+RT)/dST;
		}
		sin_a=sqrt(1.0-cos_a*cos_a);
		for(i=0;i<3;i++) {
			P[i]=RT*(cos_a*ST[i]+sin_a*P[i]);
		}
	}
	else {
		dSL=dist(S,L);
		cos_a=(RS-RL)/dSL;
		for(i=0;i<3;i++) {
			P[i]=S[i]+RS*SL[i]/cos_a-T[i];
		}
	}
	
	ang=atan(T[1]/T[0]);
	if(T[0]>0.0) {
		ang=ang+PI;
	}
	coord_ec_ge(P, ang, Time);
}

eclipse dades_eclipse_solar(int any, int mes, int dia) {
	eclipse eclip;
	eclip.n=0;
	double Time=Gregorian_to_Julian( any, mes, dia, 0, 0, 0.0);
	double Tfin=Gregorian_to_Julian( any, mes, dia, 24, 0, 0.0);
	double inc=(double)1/(24*4), inc2=(double)1/(24*60*10);
	int t=0, t2;
	
	t=eclipse_solar(Time);
	if(t>0) {
		Time=Time-1;
		t=0;
	}
	
	for(Time; Time<=Tfin&&t==0; Time=Time+inc) {
		t=eclipse_solar(Time);
	}
	
	if(t==0) {
		return eclip;
	}
	
	Time=Time-2*inc;
	t=0;
	for(Time; t==0; Time=Time+inc2) {
		t=eclipse_solar(Time);
	}
	
	contacte_eclip(Time, eclip.punt_contacte[0], 0);
	eclip.temps[eclip.n++]=Time;
	
	t=1;
	for(Time; t==1; Time=Time+inc) {
		t=eclipse_solar(Time);
	}

	if(t>1) {
		Time=Time-2*inc;
		t=1;
		for(Time; t==1; Time=Time+inc2) {
			t=eclipse_solar(Time);
		}
		eclip.tipus[eclip.n]=t;
		contacte_eclip(Time, eclip.punt_contacte[1], 1);
		eclip.temps[eclip.n++]=Time;
		t2=t;
		for(Time; t>1; Time=Time+inc) {
			t=eclipse_solar(Time);
			if(t!=t2&&t>1&&eclip.n!=7) {
				Time=Time-inc;
				t=t2;
				for(Time; t==t2; Time=Time+inc2) {
					t=eclipse_solar(Time);
				}
				eclip.tipus[eclip.n]=t;
				contacte_eclip(Time, eclip.punt_contacte[eclip.n], 2);
				eclip.temps[eclip.n++]=Time;
				t2=t;
			}
		}
		eclip.tipus[eclip.n]=1;
		Time=Time-inc;
		for(Time; t>1; Time=Time+inc2) {
			t=eclipse_solar(Time);
		}
		contacte_eclip(Time, eclip.punt_contacte[eclip.n], 1);
		eclip.temps[eclip.n++]=Time;
		for(Time; t!=0; Time=Time+inc) {
			t=eclipse_solar(Time);
		}
	}
	Time=Time-2*inc;
	t=1;
	for(Time; t>0; Time=Time+inc2) {
		t=eclipse_solar(Time);
	}
	contacte_eclip(Time, eclip.punt_contacte[eclip.n], 0);
	eclip.temps[eclip.n]=Time;
	
	return eclip;
}

eclipse dades_eclipse_lunar(int any, int mes, int dia) {
	eclipse eclip;
	eclip.n=0;
	double Time=Gregorian_to_Julian( any, mes, dia, 0, 0, 0.0);
	double Tfin=Gregorian_to_Julian( any, mes, dia, 24, 0, 0.0);
	double inc=(double)1/(24*4), inc2=(double)1/(24*60*10);
	int t=0, t2;
	
	t=eclipse_lunar(Time);
	if(t>0) {
		Time=Time-1;
		t=0;
	}
	
	for(Time; Time<=Tfin&&t==0; Time=Time+inc) {
		t=eclipse_lunar(Time);
	}
	
	if(t==0) {
		return eclip;
	}
	
	Time=Time-2*inc;
	t=0;
	for(Time; t==0; Time=Time+inc2) {
		t=eclipse_lunar(Time);
	}
	eclip.temps[eclip.n++]=Time;
	
	t=1;
	for(Time; t==1; Time=Time+inc) {
		t=eclipse_lunar(Time);
	}

	if(t>1) {
		eclip.tipus[eclip.n]=t;
		Time=Time-2*inc;
		t=1;
		for(Time; t==1; Time=Time+inc2) {
			t=eclipse_lunar(Time);
		}
		eclip.temps[eclip.n++]=Time;
		
		for(Time; t==2; Time=Time+inc) {
			t=eclipse_lunar(Time);
		}
		if(t>2) {
			eclip.tipus[eclip.n]=t;
			Time=Time-2*inc;
			t=2;
			for(Time; t==2; Time=Time+inc2) {
				t=eclipse_lunar(Time);
			}
			eclip.temps[eclip.n++]=Time;
			
			for(Time; t==3; Time=Time+inc) {
				t=eclipse_lunar(Time);
			}
			Time=Time-2*inc;
			t=3;
			for(Time; t==3; Time=Time+inc2) {
				t=eclipse_lunar(Time);
			}
			eclip.temps[eclip.n++]=Time;
			for(Time; t>1; Time=Time+inc) {
				t=eclipse_lunar(Time);
			}
		}
		Time=Time-2*inc;
		t=2;
		for(Time; t>1; Time=Time+inc2) {
			t=eclipse_lunar(Time);
		}
		eclip.temps[eclip.n++]=Time;
		
		for(Time; t>0; Time=Time+inc) {
			t=eclipse_lunar(Time);
		}
	}
	Time=Time-2*inc;
	t=1;
	for(Time; t>0; Time=Time+inc2) {
		t=eclipse_lunar(Time);
	}
	eclip.temps[eclip.n]=Time;

	return eclip;
}

int contacte_horitzo(double Time, double P[2][3], int tipus) {
	int i;
	double S[3], T[3], L[3], Q[3];
	double l, dSL, dST, dSQ, dQT, r, h;
	double a, TSL, ang;
	double SL[3], ST[3], X[3], Y[3];
	
	pos_STL(Time,S,T,L);
	
	dSL=dist(S,L);
	if(tipus==0) {
		a=asin((RS+RL)/dSL);
	}
	else {
		a=asin((RS-RL)/dSL);
	}
	
	dST=dist(S,T);
	vect_unit(S,L,SL);
	vect_unit(S,T,ST);
	TSL=acos(prod_esc(SL,ST));
	dSQ=dST*cos(TSL);
	dQT=dST*sin(TSL);
	l=tan(a)*(dSQ-RS/sin(a));
	
	if(l+RT<dQT) {
		return -1;
	}
	
	for(i=0;i<3;i++) {
		Q[i]=S[i]+dSQ*SL[i];
		X[i]=T[i]-Q[i];
	}
	norm(X);
	prod_vect(SL,X,Y);
	ang=atan(T[1]/T[0]);
	if(T[0]>0.0) {
		ang=ang+PI;
	}
	
	r=(dQT*dQT+l*l-RT*RT)/(2*dQT);
	h=sqrt(l*l-r*r);
	for(i=0;i<3;i++) {
		P[0][i]=Q[i]+r*X[i]-h*Y[i]-T[i];
		P[1][i]=Q[i]+r*X[i]+h*Y[i]-T[i];
		P[2][i]=Q[i]+r*X[i]-T[i];
	}
	coord_ec_ge(P[0], ang, Time);
	coord_ec_ge(P[1], ang, Time);
	coord_ec_ge(P[2], ang, Time);
	
	if(l+P[0][0]>=dQT) {
		r=(dQT*dQT+l*l-P[0][0]*P[0][0])/(2*dQT);
		h=sqrt(l*l-r*r);
		for(i=0;i<3;i++) {
			P[0][i]=Q[i]+r*X[i]-h*Y[i]-T[i];
		}
		coord_ec_ge(P[0], ang, Time);
	}
	else {
		return -1;
	}
	if(l+P[1][0]<dQT) {
		printf("error contacte_horitzo\n");
		return -1;
	}
		
	r=(dQT*dQT+l*l-P[1][0]*P[1][0])/(2*dQT);
	h=sqrt(l*l-r*r);
	for(i=0;i<3;i++) {
		P[1][i]=Q[i]+r*X[i]+h*Y[i]-T[i];
	}
	coord_ec_ge(P[1], ang, Time);
	
	r=(dQT*dQT+l*l-P[2][0]*P[2][0])/(2*dQT);
	for(i=0;i<3;i++) {
		P[2][i]=Q[i]+r*X[i]-T[i];
	}
	coord_ec_ge(P[2], ang, Time);
	
	return 0;
}

int eclipsi_horitzo(double Time, int n, double P[][3], int tipus) {
	int i, j;
	double S[3], T[3], L[3], Q[3], P2[3];
	double l, dSL, dST, dSQ, dQT, r, h;
	double a, TSL, ang, alpha1, alpha2, b;
	double SL[3], ST[3], X[3], Y[3]={0.0,0.0,1.0};
	
	pos_STL(Time,S,T,L);
	
	dSL=dist(S,L);
	if(tipus==0) {
		a=asin((RS+RL)/dSL);
	}
	else {
		a=asin((RS-RL)/dSL);
	}
	
	dST=dist(S,T);
	vect_unit(S,L,SL);
	vect_unit(S,T,ST);
	TSL=acos(prod_esc(SL,ST));
	dSQ=dST*cos(TSL);
	dQT=dST*sin(TSL);
	l=tan(a)*(dSQ-RS/sin(a));
		
	if(l+RT<=dQT||l+dQT<=RT) {
		return -1;
	}
	
	for(i=0;i<3;i++) {
		Q[i]=S[i]+dSQ*SL[i];
		X[i]=T[i]-Q[i];
	}
	norm(X);
	prod_vect(SL,X,Y);
	norm(Y);
	ang=atan(T[1]/T[0]);
	if(T[0]>0.0) {
		ang=ang+PI;
	}
	
	r=(dQT*dQT+l*l-RT*RT)/(2*dQT);
	h=sqrt(l*l-r*r);
	for(i=0;i<3;i++) {
		P[0][i]=Q[i]+r*X[i]+h*Y[i]-T[i];
		P[1][i]=Q[i]+r*X[i]-h*Y[i]-T[i];
	}
	coord_ec_ge(P[0], ang, Time);
	coord_ec_ge(P[1], ang, Time);
	
	if(l+P[0][0]<=dQT||l+P[1][0]<=dQT) {
		return -1;
	}
	
	r=(dQT*dQT+l*l-P[0][0]*P[0][0])/(2*dQT);
	h=sqrt(l*l-r*r);
	for(i=0;i<3;i++) {
		P[0][i]=Q[i]+r*X[i]+h*Y[i]-T[i];
	}
	coord_ec_ge(P[0], ang, Time);
	alpha1=atan(h/(dQT-r));

	r=(dQT*dQT+l*l-P[1][0]*P[1][0])/(2*dQT);
	h=sqrt(l*l-r*r);
	for(i=0;i<3;i++) {
		P2[i]=Q[i]+r*X[i]-h*Y[i]-T[i];
	}
	coord_ec_ge(P2, ang, Time);
	alpha2=atan(-h/(dQT-r));
	
	j=1;
	b=2*PI/n;
	a=floor(alpha1/b)*b;
	for(a;a>alpha2;a=a-b) {
		for(i=0;i<3;i++) {
			P[j][i]=-P[j-1][0]*cos(a)*X[i]+P[j-1][0]*sin(a)*Y[i];
		}
		coord_ec_ge(P[j], ang, Time);
		r=P[j][0];
		for(i=0;i<3;i++) {
			P[j][i]=-r*cos(a)*X[i]+r*sin(a)*Y[i];
		}
		coord_ec_ge(P[j], ang, Time);
		j++;
	}
	for(i=0;i<3;i++) {
		P[j][i]=P2[i];
	}

	return j+1;
}

//tipus=0: Eclipse parcial
//tipus=1: Eclipse total o anular

int punts_contacte(double Time, int n, double punts[][3], int tipus) {
	double S[3], T[3], L[3], V[3], Q[3];
	double SL[3], ST[3], X[3], Y[3], VP[3];
	double alpha, beta, TSL;
	double dSL, dSV, dST, dSQ, dQT, dVQ, l, r, h;
	double a, b, c, arrel, sig=-1.0;
	int i, j=0, t=0;

	pos_STL(Time,S,T,L);
	
	dSL=dist(S,L);
	vect_unit(S,L,SL);
	if(tipus==0) {
		alpha=asin((RS+RL)/dSL);
	}
	else {
		alpha=asin((RS-RL)/dSL);
	}
	dSV=RS/sin(alpha);
	for(i=0;i<3;i++) {
		V[i]=S[i]+dSV*SL[i];
	}
	
	dST=dist(S,T);
	vect_unit(S,T,ST);
	TSL=acos(prod_esc(SL,ST));
	dSQ=dST*cos(TSL);
	dQT=dST*sin(TSL);
	
	for(i=0;i<3;i++) {
		Q[i]=S[i]+dSQ*SL[i];
		X[i]=T[i]-Q[i];
	}
	norm(X);
	prod_vect(SL,X,Y);
	
	dVQ=dist(V,Q);
	l=dVQ*atan(alpha);
	
	if(l+RT<dQT) {
		return 0;
	}
	
	if(dQT+l>RT) {
		r=(dQT*dQT+l*l-RT*RT)/(2*dQT);
		h=sqrt(l*l-r*r);
		beta=atan(h/r);
		if(beta<0.0) {
			beta+=PI;
		}
	}
	else {
		beta=PI;
	}
	
	if(dSV>dSL) {
		sig=1.0;
		for(i=0;i<3;i++) {
			Q[i]=V[i]+(RS*sin(alpha)-dSV)*SL[i];
		}
		l=RS*cos(alpha);
	}
	
	for(alpha=-beta;alpha<beta&&j<n;alpha+=2*PI/n) {
		for(i=0;i<3;i++) {
			VP[i]=Q[i]+l*cos(alpha)*X[i]+l*sin(alpha)*Y[i]-V[i];
		}
		a=VP[0]*VP[0]+VP[1]*VP[1]+pow(VP[2]*RT/RTp,2);
		b=2*(VP[0]*(V[0]-T[0])+VP[1]*(V[1]-T[1])+VP[2]*(V[2]-T[2])*pow(RT/RTp,2));
		c=pow(V[0]-T[0],2)+pow(V[1]-T[1],2)+pow((V[2]-T[2])*RT/RTp,2)-RT*RT;
		arrel=b*b-4*a*c;
		if(arrel>=0.0) {
			arrel=(-b+sig*sqrt(arrel))/(2*a);
			for(i=0;i<3;i++) {
				punts[j][i]=V[i]+arrel*VP[i];
			}
			j++;
		}
	}
	if(tipus==1) {
		for(i=0;i<3;i++) {
			VP[i]=Q[i]-V[i];
		}
		a=VP[0]*VP[0]+VP[1]*VP[1]+pow(VP[2]*RT/RTp,2);
		b=2*(VP[0]*(V[0]-T[0])+VP[1]*(V[1]-T[1])+VP[2]*(V[2]-T[2])*pow(RT/RTp,2));
		c=pow(V[0]-T[0],2)+pow(V[1]-T[1],2)+pow((V[2]-T[2])*RT/RTp,2)-RT*RT;
		arrel=b*b-4*a*c;
		if(arrel>=0.0) {
			arrel=(-b+sig*sqrt(arrel))/(2*a);
			for(i=0;i<3;i++) {
				punts[n][i]=V[i]+arrel*VP[i];
			}
		}
		else {
			for(i=0;i<3;i++) {
				punts[n][i]=0.0;
			}
		}
	}
	return j;
}

int eclipse_solar_lloc(double Time, double P[3]) {
	double S[3], T[3], L[3], U[3], V[3]; //punts
	double SL[3], ST[3], PS[3], PL[3]; //vectors unitaris
	double dSL, dST, dSU, dSV; //distancies
	double aL, bL; //angles (a angle recta tangent ext. i r, i b angle recta tang. int. i r. On r s la recta entre els centres)
	double ang, aux;
	double VP[3],UP[3];
	int i;
	
	pos_STL(Time,S,T,L);
	
	ang=atan(T[1]/T[0]);
	if(T[0]>0.0) {
		ang=ang+PI;
	}
	coord_ge_ec(P, ang, Time);
	
	for(i=0;i<3;i++) {
		ST[i]=T[i]-S[i];
	}
	aux=modul(P);
	if(prod_esc(ST,P)/aux>(RS+RT)) {
		return 0;
	}
	
	for(i=0;i<3;i++) {
		P[i]=T[i]+P[i];
	}
	
	dSL=dist(S,L);
	
	aL=asin((RS-RL)/dSL);
	bL=asin((RS+RL)/dSL);
	dSV=RS/sin(aL);
	dSU=RS/sin(bL);
	
	for(i=0;i<3;i++) {
		SL[i]=(L[i]-S[i])/dSL;
	}

	for(i=0;i<3;i++) {
		V[i]=S[i]+dSV*SL[i];
		U[i]=S[i]+dSU*SL[i];
		VP[i]=P[i]-V[i];
		UP[i]=P[i]-U[i];
	}
	
	aux=acos(prod_esc(SL,UP)/modul(UP));
	if(aux>bL) {
		return 0;
	}
	
	vect_unit(P,S,PS);
	vect_unit(P,L,PL);
	aux=acos(prod_esc(PS,PL));
	if(aux<max_eclip) {
		max_eclip=aux;
		t_max_eclip=Time;
	}
	
	aux=acos(prod_esc(SL,VP)/modul(VP));
	if(aux>aL&&aux<(PI-aL)) {
		return 1;
	}
	if(aux>aL) {
		return 2;
	}
	return 3;
}

//P[0]=distancia al centre de la Terra, P[1]=longitud, i P[2]=latitud
eclipse eclip_lloc(double any, double mes, double dia, double P[]) {
	double S[3], T[3], L[3], Q[3];
	double Time=Gregorian_to_Julian( any, mes, dia, 0, 0, 0.0);
	double Tfin=Gregorian_to_Julian( any, mes, dia, 24, 0, 0.0);
	double inc=(double)1/(24*60*60), ang;
	int i, t=0, t2;
	eclipse eclip;
	eclip.n=0;
	max_eclip=PI;
	
	for(i=0;i<3;i++) {
		Q[i]=P[i];
	}
	t=eclipse_solar_lloc(Time,Q);
	if(t>0) {
		Time=Time-1;
		t=0;
	}
	
	for(Time; Time<=Tfin&&t==0; Time=Time+inc) {
		for(i=0;i<3;i++) {
			Q[i]=P[i];
		}
		t=eclipse_solar_lloc(Time,Q);
	}
	
	if(t==0) {
		return eclip;
	}
	
	eclip.temps[eclip.n++]=Time;
	
	t=1;
	for(Time; t==1; Time=Time+inc) {
		for(i=0;i<3;i++) {
			Q[i]=P[i];
		}
		t=eclipse_solar_lloc(Time,Q);
	}

	if(t>1) {
		eclip.tipus[eclip.n]=t;
		eclip.temps[eclip.n++]=Time;
		t2=t;
		for(Time; t>1; Time=Time+inc) {
			for(i=0;i<3;i++) {
				Q[i]=P[i];
			}
			t=eclipse_solar_lloc(Time,Q);
			if(t!=t2&&t>1&&eclip.n!=7) {
				eclip.tipus[eclip.n]=t;
				eclip.temps[eclip.n++]=Time;
				t2=t;
			}
		}
		eclip.tipus[eclip.n]=1;
		eclip.temps[eclip.n++]=Time;
		for(Time; t!=0; Time=Time+inc) {
			for(i=0;i<3;i++) {
				Q[i]=P[i];
			}
			t=eclipse_solar_lloc(Time,Q);
		}
	}
	eclip.temps[eclip.n]=Time;
	if(eclip.n>0) {
		eclip.max_eclip=t_max_eclip;
	}

	return eclip;
}

// tipus==0: eix x pertanyent a l'horitzo (cap a l'oest)
// tipus==1: eix x pertanyent a l'equador global (cap a l'oest)
// info[0]: angle visio Sol (radi)
// info[1]: angle visio Lluna (radi)
// info[2]: angle de visi Sol-Lluna
// info[3]: angle eix Sol-Lluna respecte l'eix x a l'oest (antihorari)
void visio_eclipsi(double Time, double P[3], double info[4], int tipus) {
	double Q[3], Q2[3]={0.0,0.0,1.0}, S[3], T[3], L[3], QS[3], QL[3], X[3], Y[3];
	double ang, aux, aux2;
	int i, n=0;
	
	pos_STL(Time,S,T,L);
	ang=atan(T[1]/T[0]);
	if(T[0]>0.0) {
		ang=ang+PI;
	}
	if(tipus==0) {
		for(i=0;i<3;i++) {
			Q2[i]=P[i];
		}
	}
	coord_ge_ec(Q2, ang, Time);
	for(i=0;i<3;i++) {
		Q[i]=T[i]+Q2[i];
	}
	info[0]=asin(RS/dist(Q,S));
	info[1]=asin(RL/dist(Q,L));
	vect_unit(Q,S,QS);
	vect_unit(Q,L,QL);
	info[2]=acos(prod_esc(QS,QL));
	prod_vect(QS,Q2,X);
	prod_vect(X,QS,Y);
	norm(X);
	norm(Y);
	aux=prod_esc(QL,X);
	aux2=prod_esc(QL,Y);
	info[3]=acos(aux/sqrt(aux*aux+aux2*aux2));
	if(aux2<0) {
		info[3]+=PI;
	}
}
