#include <math.h>
#include "ephem_read.c"

/*const const double PI=3.14159265;
const double MT=5.9742e24;
const double ML=7.3483e22;
const double MS=1.9891e30;*/
extern const double PI, MT, ML, MS;

double dist(double a[3], double b[3]) {
	return sqrt(pow(a[0]-b[0],2)+pow(a[1]-b[1],2)+pow(a[2]-b[2],2));
}

double modul(double a[3]) {
	return sqrt(pow(a[0],2)+pow(a[1],2)+pow(a[2],2));
}

double norm(double a[3]) {
	int i;
	double d=modul(a);
	
	for(i=0;i<3;i++) {
		a[i]/=d;
	}
}

double prod_esc(double X[3], double Y[3]) {
	return X[0]*Y[0]+X[1]*Y[1]+X[2]*Y[2];
}

void prod_vect(double X[3], double Y[3], double Z[3]) {
	Z[0]=X[1]*Y[2]-X[2]*Y[1];
	Z[1]=X[2]*Y[0]-X[0]*Y[2];
	Z[2]=X[0]*Y[1]-X[1]*Y[0];
}

void vect_unit(double X[3], double Y[3], double XY[3]) {
	int i;
	double d=dist(X,Y);
	for(i=0; i<3; i++) {
		XY[i]=(Y[i]-X[i])/d;
	}
}

void pos_STL(double Time, double S[3], double T[3], double L[3]) {
	
	int i;
	
	Interpolate_Position( Time, 2 , T );
	Interpolate_Position( Time, 9 , L );
	Interpolate_Position( Time, 10 , S );
    for(i=0; i<3; i++) {
    	L[i]=T[i]+L[i];
	}
	for(i=0;i<3;i++) {
		T[i]=(T[i]*(MT+ML)-L[i]*ML)/MT;
	}
}

void state_sist_solar(double Time, stateType P[]) {
	int i;
	for(i=0;i<11;i++) {
		Interpolate_State( Time, i , &P[i] );
	}
    for(i=0; i<3; i++) {
    	P[9].Position[i]=P[2].Position[i]+P[9].Position[i];
    	P[9].Velocity[i]=P[2].Velocity[i]+P[9].Velocity[i];
	}
	for(i=0;i<3;i++) {
		P[2].Position[i]=(P[2].Position[i]*(MT+ML)-P[9].Position[i]*ML)/MT;
		P[2].Velocity[i]=(P[2].Velocity[i]*(MT+ML)-P[9].Velocity[i]*ML)/MT;
	}
}

//longitut positiva cap a l'est desde el meridia de grenwich
//longitut negativa cap a l'oest desde el meridia de grenwich
//X[0]=distancia al centre de la Terra, X[1]=longitud, i X[2]=latitud

void coord_ec_ge(double X[3], double ang, double Time) {
	double aux;
	aux=sqrt(pow(X[0],2)+pow(X[1],2)+pow(X[2],2));
	X[1]=atan(X[1]/X[0]);
	if(X[0]<0.0) {
		X[1]=X[1]+PI;
	}
	X[1]=X[1]-ang-(Time-floor(Time))*2*PI;
	if(X[1]<-PI) {
		X[1]=X[1]+2*PI;
	}
	else if(X[1]>PI) {
		X[1]=X[1]-2*PI;
	}
	X[0]=aux;
	X[2]=asin(X[2]/X[0]);
}

void coord_ge_ec(double X[3], double ang, double Time) {
	double aux1, aux2;
	aux1=X[1]+ang+(Time-floor(Time))*2*PI;
	aux2=X[0];
	X[0]=aux2*cos(X[2])*cos(aux1);
	X[1]=aux2*cos(X[2])*sin(aux1);
	X[2]=aux2*sin(X[2]);
}

void rotacio(double X[3], double ang, int t) {
	double aux, aux2;
	int i,j;
	
	if(t==1) {
		i=1;
		j=2;
	}
	else if(t==2) {
		i=2;
		j=0;
	}
	else {
		i=0;
		j=1;
	}
	aux=cos(ang)*X[i]-sin(ang)*X[j];
	aux2=sin(ang)*X[i]+cos(ang)*X[j];
	X[i]=aux;
	X[j]=aux2;
}
