#include <math.h>
#include "temps.h"

void gdcon(double temps, data *d) {
	char  *Month;
	double  A, B, C, D, E, F, T, Z, Alpha, Frac, Total, Whole;

  /* Add a half day to input Julian date......................................*/
  
  T = temps + 0.5;
  Z = floor(T);
  F = T - Z;

  /* Preliminary calculations.................................................*/

  Alpha = floor((Z - 1867216.25)/36524.25);
  A     = 1.0 + Z + Alpha - floor(Alpha/4.0);
  B     = A + 1524.0;
  C     = floor((B - 122.1)/365.25);
  D     = floor(365.25*C);
  E     = floor((B - D)/30.6001);

  /* Compute the month........................................................*/

  if (E<14)
     d->mes = ((int) E) - 1; 
  else 
     d->mes = ((int) E) - 13;  

  /* Compute the year.........................................................*/
  
  if (d->mes>2)
     d->any = ((int) C) - 4716;
  else
     d->any = ((int) C) - 4715;
  
  /* Compute the day..........................................................*/
  
   Total = B - D + F - floor(30.6001*E);
   Whole = floor(Total);
   Frac  = Total - Whole;
   d->dia = (int) Whole;
   
  /* Compute the hour.........................................................*/
   
   Total = Frac * 24.0;
   Whole = floor(Total);
   Frac  = Total - Whole;
   d->hora = (int) Whole;
   
  /* Compute the minute.......................................................*/
   
   Total  = Frac * 60.0;
   Whole  = floor(Total);
   Frac   = Total - Whole;
   d->min = (int) Whole;

  /* Compute the second.......................................................*/  

   d->seg = Frac * 60.0;
}

double Gregorian_to_Julian( int year, int month, int day, int hour, int min, double seconds)
{
  double  A, B, D, H, JD, M, N, Y;

  /*--------------------------------------------------------------------------*/
  /*  Adjust year & month values (if necessary).                              */
  /*--------------------------------------------------------------------------*/

  if ( month < 3 ) {                                      /* See Muess, p. 61 */
       month = month + 12;
       year  = year - 1;
     }

  /*--------------------------------------------------------------------------*/
  /*  Perform type casts and adjust the day count.                            */
  /*--------------------------------------------------------------------------*/

  Y = (double) year;
  M = (double) month;
  D = (double) day;
  H = (double) hour;
  N = (double) min;

  /*--------------------------------------------------------------------------*/
  /*  Compute the day fraction:                                               */
  /*--------------------------------------------------------------------------*/

  D = D  +  (H / 24.0)  +  (N / 1440.0)  +  (seconds / 86400.0);

  /*--------------------------------------------------------------------------*/
  /*  Compute the Julian date.                                                */
  /*--------------------------------------------------------------------------*/

  A  = floor(Y/100.0);
  B  = 2.0 - A + floor(A/4.0);  
  JD = floor(365.25*(Y+4716.0)) + floor(30.6001*(M+1.0)) + D + B - 1524.5;
  
  return JD;
}
